@@ -1,4 +1,4 @@
-from .exception import PipelineException
+from .exception import PipelineError
 from .message import Message
 from .tap import DestinationTap, SourceTap, SourceOf, DestinationOf
 from .worker import Generator, Processor, Splitter
@@ -6,7 +6,7 @@ from .utils import parse_kind
 
 
 __all__ = [
-    'PipelineException',
+    'PipelineError',
     'SourceOf',
     'DestinationOf',
     'SourceTap',
