@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-import codecs
 import json
 import logging
 import os
@@ -167,7 +166,7 @@ class FileSource(SourceTap):
     def __init__(self, config, logger=logger):
         super().__init__(config, logger)
         if config.infile == '-':
-            self.infile = codecs.getreader('utf-8')(sys.stdin)
+            self.infile = sys.stdin
         else:
             self.infile = open(config.infile, 'r')
         self.repeat = config.repeat
@@ -187,7 +186,8 @@ class FileSource(SourceTap):
         for i in range(0, self.repeat):
             for line in self.infile:
                 yield self.messageClass(line)
-            self.infile.seek(0, 0)
+            if self.repeat > 1:
+                self.infile.seek(0, 0)
 
 
 class FileDestination(DestinationTap):
@@ -207,8 +207,9 @@ class FileDestination(DestinationTap):
 
     def __init__(self, config, logger=logger):
         super().__init__(config, logger)
+        self.filename = config.outfile
         if config.outfile == '-':
-            self.outFile = codecs.getwriter('utf-8')(sys.stdout)
+            self.outFile = sys.stdout
         else:
             self.outFile = open(config.outfile, 'w')
 
@@ -222,11 +223,11 @@ class FileDestination(DestinationTap):
                             help='output json file')
 
     def write(self, message):
-        self.outFile.write(message.serialize().decode('utf-8'))
-        self.outFile.write('\n')
+        print(message.serialize().decode('utf-8'), file=self.outFile)
 
     def close(self):
-        self.outFile.close()
+        if self.filename != '-':
+            self.outFile.close()
 
 
 class KafkaSource(SourceTap):
